##********************************************************************************************************************
##TOOL NAME: Curvature
##SOURCE NAME: Curvature.py
##VERSION: 1
##AUTHOR: B.M.J. Possel (b.m.j.possel@student.tudelft.nl
##REQUIRED ARGUMENTS: Input raster, Output folder, Window Size, Skip pixels
##
##TOOL DESCRIPTION: Manipulates a Raster map and writes them to a selected Folder.
##
##DATE: February 23, 2009
##
##Usage: Curvature <Input_Raster_Dataset> <Output_Folder> <Window_Size> <Skip pixel>
##
##Code inspired by Matthew Perry's "slope.cpp" and Julien Fiore's "openness.py"
##*********************************************************************************************************************

import sys, string, os, time
try:
    import win32com.client, pythoncom
except ImportError:
    print "Install PyWin32: http://www.python.org/download/windows/"
try:
    from numpy import *
    from numpy.linalg import inv
except ImportError:
    print "Install Numpy: www.numpy.org"
try:
    from osgeo import gdal
except ImportError:
    print "Install gdal: www.gdal.org"
import padding          #Load padding script

msgSuccess="Successfully built file "
msgFail="Failed to build file "

gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")
gp.SetProduct("ArcView")

# Fitting polynomial using a least-squares approach
def FitCurv(T2, y2):
    xdak = dot(dot(inv(dot(T2.conj().transpose(),T2)),T2.conj().transpose()),y2)#Check if this is fully correct!!!
    return xdak

def CalculateCurvature(InFile,OutFile,size2,skippixel):
    # Open inGrid
    inGrid = gdal.Open(InFile)
    inGridBand = inGrid.GetRasterBand(1) # get raster band

    # Get inGrid parameters
    geoTrans = inGrid.GetGeoTransform() # returns list (xOrigin,xcellsize,...)

    cellSizeX = geoTrans[1]
    cellSizeY = geoTrans[5]
    nXSize = inGrid.RasterXSize
    nYSize = inGrid.RasterYSize
    
    #nullValue = inGridBand.GetNoDataValue() #For some reason it doesn't always work...
    nullValue2 = -999

    # Create  output file
    format = "GTiff"
    driver = gdal.GetDriverByName( format )
    outGrid=driver.CreateCopy(OutFile,inGrid)   # Creates output raster with same properties as input
    outGridBand = outGrid.GetRasterBand(1)      # Access Band 1
    outGridBand.SetNoDataValue(nullValue2)

    #min = inGridBand1.GetMinimum()
    #max = inGridBand1.GetMaximum()
    #print 'Min=%.3f, Max=%.3f' % (min,max)

    # Matrix Manupulation
    matrix2 = inGridBand.ReadAsArray()
    nullValue = matrix2.min()
    [p,q] = shape(matrix2)              # Used for determining tile size

    [matrix, basemodel] = padding.padding(InFile,size2)                #Add padding from neighbours

    print 'matrix'
    print matrix[0,0]
    print matrix.dtype
    
    # Fitting planes
    [s,t] = shape(matrix)
    Curv = ones((s,t),float)*nullValue2

    a1fit = zeros((s,t),float)
    a2fit = zeros((s,t),float)
    a3fit = zeros((s,t),float)
    a4fit = zeros((s,t),float)
    a5fit = zeros((s,t),float)
    a6fit = zeros((s,t),float)
    
    n = range(-size2,(size2+1))
    [X,Y] = meshgrid(n,n)
    X2 = X[0:(len(X)):skippixel,0:(len(X)):skippixel]
    Y2 = Y[0:(len(Y)):skippixel,0:(len(Y)):skippixel]
    
    Xflat = X2.flatten(1)[:,newaxis]
    Yflat = Y2.flatten(1)[:,newaxis]
   
    T = ones(((len(Xflat)),3),float)
    T[:,0] = Xflat[:,0]
    T[:,1] = Yflat[:,0]

    L = len(X2)
    M = (L-1)/2

    A = L**2
    B = (M*(M+1)*(L**2))/3
    C = (M*(M+1)*(4*M**2+4*M-3)*(L**2))/45
    D = (M**2*((M+1)**2)*(L**2))/9

    print L
    print M
    print A
    print B
    print C
    print D

    for i in range(s)[(size2):(s-(size2+1))]:
        for j in range(t)[(size2):(t-(size2+1))]:
            if matrix[i,j] != nullValue:
                plane = matrix[(i-size2):(i+size2+1):skippixel,(j-size2):(j+size2+1):skippixel]
                planeflat = plane.flatten(1)[:,newaxis]

                T[:,2] = planeflat[:,0]
                TnoNaN = T[where(T[:,2]!=nullValue)[0],:]  # extract the columms where value = nullValue 
                #TnoNaN is a float64 array -> why? usefull?

                if len(TnoNaN) > 30:
                    y = TnoNaN[:,2].copy()
                    TnoNaN2 = zeros((len(TnoNaN),6),float)
                    TnoNaN2[:,0] = 1
                    TnoNaN2[:,1] = TnoNaN[:,0].copy()
                    TnoNaN2[:,2] = TnoNaN[:,1].copy()
                    TnoNaN2[:,3] = TnoNaN[:,0].copy() * TnoNaN[:,0].copy() -M*(M+1)/3
                    TnoNaN2[:,4] = TnoNaN[:,0].copy() * TnoNaN[:,1].copy()
                    TnoNaN2[:,5] = TnoNaN[:,1].copy() * TnoNaN[:,1].copy() -M*(M+1)/3

                    try:
                        xdak = FitCurv(TnoNaN2,y)
                    except:
                        print gp.GetMessages()
                    
                    a1fit[i,j] = xdak[0]/A
                    a2fit[i,j] = xdak[1]/B
                    a3fit[i,j] = xdak[2]/B
                    a4fit[i,j] = xdak[3]/C
                    a5fit[i,j] = xdak[4]/D
                    a6fit[i,j] = xdak[5]/C
                
                    Curv[i,j] = - ((1+(a3fit[i,j]**2))*(2*a4fit[i,j]) - 2*a2fit[i,j]*a3fit[i,j]*a5fit[i,j] + (1+(a2fit[i,j]**2))*2*a6fit[i,j]) / 2*(1 + (a2fit[i,j]**2 + a3fit[i,j]**2))**1.5
                    Curv[i,j] = Curv[i,j] * 10000000

                    #kappa[i,j] = ((matrixa[i,j]*xdak[1,])-(matrixb[i,j]*xdak[0,]))/(((matrixa[i,j]**2+matrixb[i,j]**2))**1.5)
                    #Curv[i,j] = ((xdak[0,]*matrixb[i,j])-(2*conv*matrix[i,j])+(xdak[1,]*matrixa[i,j]))/(2*(matrixa[i,j]*matrixb[i,j]-matrix[i,j]**2))
                     
    # Writes buffer to file
    Curv2 = Curv[size2:(p+size2),size2:(q+size2)]
    
    print 'Curv Matrix'
    print Curv2.max()
    print Curv2.min()
    print Curv2[0,0]
    print Curv2.dtype
    time.sleep(5)
    
    outGridBand.WriteArray(Curv2)
    outGridBand.FlushCache()

#The input rasters
rasterlist = gp.GetParameterAsText(0)

#Remove the parathesis around each input featureclass
rasterlist = string.replace(rasterlist, "(", "")
rasterlist = string.replace(rasterlist, ")", "")

#split input raster datasets by semicolon ";"
in_raster_ds = string.split(rasterlist, ";")
    
for in_raster in in_raster_ds:
    raster = in_raster

    if raster.startswith("'") and raster.endswith("'"):
        raster = raster[1:-1]

    #Determine folder and filename
    OutFolder = gp.GetParameterAsText(1)
    OutFolder = string.replace(OutFolder, "\\", "/")
    OutFile = OutFolder + "/" + os.path.basename(raster) + "_curv.tif"
    raster = string.replace(raster, "\\", "/")
    #### TO ADD -> Als .adf niet bestaat, neem dan het input bestand!!!
    #InFile = raster + "/" + "w001001.adf"

    InFile = raster
    InFile = InFile.encode('utf8')
    OutFile = OutFile.encode('utf8')

    print "Files:"
    print InFile
    print OutFile

    size2 = int(gp.GetParameterAsText(2)) #Size of the plain
    if size2 < 1:
        print "Change Input Parameters: Size >= 1"
        time.sleep(5)
        sys.exit()

    skippixel = int(gp.GetParameterAsText(3)) #Amount of pixels to skip, uses every x-number of pixels
    if skippixel < 1:
        print "Change Input Parameters: Skip pixel >= 1"
        time.sleep(5)
        sys.exit()

    try:
        CalculateCurvature(InFile,OutFile,size2,skippixel)
    except:
        # If an error occurred while running a tool, then print the messages.
        print gp.GetMessages()

    gp.AddMessage(msgSuccess + "%s" % raster)

        
#############################################################
