##********************************************************************************************************************
##TOOL NAME: Density
##SOURCE NAME: Density.py
##VERSION: 1.0
##AUTHOR: B.M.J. Possel (b.m.j.possel@student.tudelft.nl
##REQUIRED ARGUMENTS: Input raster, Output folder, Plain size, Minimum # of points
##
##TOOL DESCRIPTION: Filters a Raster map based on point density, writes created maps to a selected Folder.
##
##DATE: July 21, 2009
##
##Usage: Density <Input_Raster_Dataset> <Output_Folder> <Plain_Size> <Point_Number>
##
##*********************************************************************************************************************


# Import

import sys, string, os, time
try:
    import win32com.client, pythoncom
except ImportError:
    print "Install PyWin32: http://www.python.org/download/windows/"
try:
    from numpy import *
    from numpy.linalg import inv
except ImportError:
    print "Install Numpy: www.numpy.org"
try:
    from osgeo import gdal
except ImportError:
    print "Install gdal: www.gdal.org"
import padding          #Load padding script
#import FitPlaneprocess       #Load FitPlane script

msgSuccess="Successfully built file "
msgFail="Failed to build file "

gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")
gp.SetProduct("ArcView")


def CalcDensity(InFile, OutFile1, size2, minpixels):
    # Open inGrid
    inGrid = gdal.Open(InFile)
    inGridBand = inGrid.GetRasterBand(1) # get raster band

    # Get inGrid parameters
    geoTrans = inGrid.GetGeoTransform() # returns list (xOrigin,xcellsize,...)

    cellSizeX = geoTrans[1]
    cellSizeY = geoTrans[5]
    nXSize = inGrid.RasterXSize
    nYSize = inGrid.RasterYSize
    
    #nullValue = inGridBand.GetNoDataValue() #For some reason it doesn't always work...
    nullValue2 = -999

    # Create  output file
    format = "GTiff"
    driver = gdal.GetDriverByName( format )
    outGrid1=driver.CreateCopy(OutFile1,inGrid)   # Creates output raster with same properties as input
    outGridBand1 = outGrid1.GetRasterBand(1)      # Access Band 1
    outGridBand1.SetNoDataValue(nullValue2)

    min = inGridBand.GetMinimum()
    max = inGridBand.GetMaximum()
    #print 'Min=%.3f, Max=%.3f' % (min,max)

    # Matrix Manupulation
    matrix2 = inGridBand.ReadAsArray()
    nullValue = matrix2.min()
    [p,q] = shape(matrix2)                        # Used for determining tile size

    [matrix, basemodel] = padding.padding(InFile,size2)                #Add padding from neighbours

    print 'matrix'
    print matrix[0,0]
    print matrix.dtype

    [s,t] = shape(matrix)
    filtered = ones((s,t),float)*nullValue2
  
    for i in range(s)[(size2):(s-(size2+1))]:
        for j in range(t)[(size2):(t-(size2+1))]:
            if matrix[i,j] != nullValue:
                plane = matrix[(i-size2):(i+size2+1):1,(j-size2):(j+size2+1):1]
                planeflat = plane.flatten(1)[:,newaxis]
                TnoNaN = planeflat[where(planeflat[:,0]!=nullValue)[0],:]  # extract the columms where value = nullValue 
                #print plane
                #print planeflat
                #print TnoNaN               
                
                if len(TnoNaN) > minpixels:
                    filtered[i,j] = matrix[i,j]
                    #print minpixels
                    #print len(TnoNaN)
                    #time.sleep(3)

    print filtered.max()
    print filtered.min()

    # Writes buffer to file
    filtered2 = filtered[size2:(p+size2),size2:(q+size2)]

    print 'matrix2'
    print filtered2[0,0]
    print filtered2.dtype
    time.sleep(5)
    
    outGridBand1.WriteArray(filtered2)
    outGridBand1.FlushCache()

#The input rasters
rasterlist = gp.GetParameterAsText(0)

#Remove the parathesis around each input featureclass
rasterlist = string.replace(rasterlist, "(", "")
rasterlist = string.replace(rasterlist, ")", "")

#split input raster datasets by semicolon ";"
in_raster_ds = string.split(rasterlist, ";")
    
for in_raster in in_raster_ds:
    raster = in_raster

    if raster.startswith("'") and raster.endswith("'"):
        raster = raster[1:-1]

    #Determine folder and filename
    OutFolder = gp.GetParameterAsText(1)
    OutFolder = string.replace(OutFolder, "\\", "/")

    BaseName = os.path.basename(raster)
    SplitBaseName = string.split(BaseName, ".")
    OutFile1 = OutFolder + "/" + SplitBaseName[0] + "2.tif"
    #+ "2" + os.path.basename(raster)
    raster = string.replace(raster, "\\", "/")
    #### TO ADD -> Als .adf niet bestaat, neem dan het input bestand!!!
    #InFile = raster + "/" + "w001001.adf"
    InFile = raster

    InFile = InFile.encode('utf8')
    OutFile1 = OutFile1.encode('utf8')

    size2 = int(gp.GetParameterAsText(2)) #Size of the plain
    if size2 < 1:
        print "Change Input Parameters: Size >= 1"
        time.sleep(5)
        sys.exit()

    density = int(gp.GetParameterAsText(3)) #Density of pixels inside plain
    if density < 0:
        print "Change Input Parameters: Minimum percentage of pixels = 1"
        time.sleep(5)
        sys.exit()
    if density > 100:
        print "Change Input Parameters: Maximum percentage of pixels = 100"
        time.sleep(5)
        sys.exit()
    minpixels = around(density*((size2+size2+1)**2)/100)
    print minpixels
    time.sleep(5)

    CalcDensity(InFile,OutFile1,size2,minpixels)
    gp.AddMessage(msgSuccess + "%s" % raster)

        
#############################################################
