##********************************************************************************************************************
##TOOL NAME: FitPlane
##SOURCE NAME: FitPlane.py
##VERSION: 1.1
##AUTHOR: B.M.J. Possel (b.m.j.possel@student.tudelft.nl
##REQUIRED ARGUMENTS: Input raster, Output folder
##
##TOOL DESCRIPTION: Manipulates a Raster map and writes them to a selected Folder.
##
##DATE: July 15, 2008
##
##Usage: FitPlane <Input_Raster_Dataset> <Output_Folder> 
##
##Code inspired by Matthew Perry's "slope.cpp" and Julien Fiore's "openness.py"
##*********************************************************************************************************************


# Import

import sys, string, os, time
try:
    import win32com.client, pythoncom
except ImportError:
    print "Install PyWin32: http://www.python.org/download/windows/"
try:
    from numpy import *
    from numpy.linalg import inv
except ImportError:
    print "Install Numpy: www.numpy.org"
try:
    from osgeo import gdal
except ImportError:
    print "Install gdal: www.gdal.org"

msgSuccess="Successfully built file "
msgFail="Failed to build file "

gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")
gp.SetProduct("ArcView")

# Check out Spatial Analyst extension license
gp.CheckOutExtension("Spatial")


def Filter(InFile, OutFile, size1, VarThreshold, size2, SlopeThreshold):
    # Open inGrid
    inGrid = gdal.Open(InFile)
    inGridBand = inGrid.GetRasterBand(1) # get raster band

    # Get inGrid parameters
    geoTrans = inGrid.GetGeoTransform() # returns list (xOrigin,xcellsize,...)

    cellSizeX = geoTrans[1]
    cellSizeY = geoTrans[5]
    nXSize = inGrid.RasterXSize
    nYSize = inGrid.RasterYSize
    nullValue = inGridBand.GetNoDataValue()
    nullValue2 = -999

    # Create  output file
    format = "GTiff"
    driver = gdal.GetDriverByName( format )
    outGrid=driver.CreateCopy(OutFile,inGrid)   # Creates output raster with same properties as input
    outGridBand = outGrid.GetRasterBand(1)      # Access Band 1
    outGridBand.SetNoDataValue(nullValue2)

    # Matrix Manupulation
    matrix2 = inGridBand.ReadAsArray()
    matrix = 100*  matrix2.copy()    #NAAR KIJKEN!!!
    matrix = (matrix.astype('int'))  #NAAR KIJKEN!!!
    matrix[matrix<-1000]=-999
    
    [s,t] = shape(matrix)
    filt = ones((s,t),float)*nullValue2
    variance = ones((s,t),float)*nullValue2
    slope = ones((s,t),float)*nullValue2

    size = max(size1, size2)
    for i in range(s)[(size):(s-(size+1))]:
        for j in range(t)[(size):(t-(size+1))]:
            if matrix[i,j] != nullValue2:
                plain1 = matrix[(i-size1):(i+size1+1),(j-size1):(j+size1+1)]
                plain2 = matrix[(i-size2):(i+size2+1),(j-size2):(j+size2+1)]

                plainflat1 = plain1.flatten(1)[:,newaxis]
                plainflat1noNaN = plainflat1[where(plainflat1[:,0]!=nullValue2)[0],:]

                if plainflat1noNaN.size >= 3:
                    if var(plainflat1noNaN) <VarThreshold:
                        filt[i,j]=matrix[i,j]

    print 'Min=%.3f, Max=%.3f' % (variance.min(),variance.max())
    time.sleep(5)
    
    outGridBand.WriteArray(filt)
    outGridBand.FlushCache()

#The input rasters
rasterlist = gp.GetParameterAsText(0)

#Remove the parathesis around each input featureclass
rasterlist = string.replace(rasterlist, "(", "")
rasterlist = string.replace(rasterlist, ")", "")

#split input raster datasets by semicolon ";"
in_raster_ds = string.split(rasterlist, ";")
    
for in_raster in in_raster_ds:
    raster = in_raster

    if raster.startswith("'") and raster.endswith("'"):
        raster = raster[1:-1]

    #Determine folder and filename
    OutFolder = gp.GetParameterAsText(1)
    OutFolder = string.replace(OutFolder, "\\", "/")
    OutFile = OutFolder + "/" + os.path.basename(raster) + "_filtered1.tif"
    OutFile2 = OutFolder + "/" + os.path.basename(raster) + "_filtered2.tif"    
    raster = string.replace(raster, "\\", "/")
    InFile = raster + "/" + "w001001.adf"

    InFile = InFile.encode('utf8')
    OutFile = OutFile.encode('utf8')

    size1 = int(gp.GetParameterAsText(2)) #Size of the plain
    if size1 < 1:
        print "Change Input Parameters: Size >= 1"
        time.sleep(5)
        sys.exit()
    VarThreshold = int(gp.GetParameterAsText(3))

    size2 = int(gp.GetParameterAsText(4)) #Size of the plain
    if size2 < 1:
        print "Change Input Parameters: Size >= 1"
        time.sleep(5)
        sys.exit()
    SlopeThreshold = int(gp.GetParameterAsText(5))

    askInterpolate = gp.GetParameterAsText(7)

    InMask = gp.GetParameterAsText(6)

    print "input variables"
    print size1
    print VarThreshold
    print size2
    print SlopeThreshold
    print askInterpolate

    print "mask variables"
    print in_raster
    print OutFile
    print OutFile2
    time.sleep(5)

    try:
        ##This section is for variance filter!
        Filter(InFile, OutFile, size1, VarThreshold, size2, SlopeThreshold)
    except:
        ## If an error occurred while running a tool, then print the messages.
        print gp.GetMessages()

    try:
        ##This section is for mask extraction!
        #ExtractByMask_sa <in_raster> <in_mask_data> <OutFile>
        gp.ExtractByMask_sa(OutFile, InMask, OutFile2)
    except:
        ## If an error occurred while running a tool, then print the messages.
        print gp.GetMessages()


##STILL TO IMPLEMENT: INTERPOLATION!!!


##    if askInterpolate = 1:
##        print "Interpolating..."
##        time.sleep(5)
##
##    try:
##        # Set local variables
##        InRaster = "C:/data/ras_1"
##        InTrueRaster = "C:/data/ras_0"
##        InFalseRaster = "C:/data/ras_2"
##        OutRaster = "C:/data/out_Con"
##
##        # Check out Spatial Analyst extension license
##        gp.CheckOutExtension("Spatial")
##
##        # Process: Con
##        gp.Con_sa(InRaster, InTrueRaster, OutRaster, InFalseRaster, "VALUE = nullValue2")
##
##    except:
##        # If an error occurred while running a tool, then print the messages.
##        print gp.GetMessages()
##



    gp.AddMessage(msgSuccess + "%s" % raster)

        
#############################################################
