##********************************************************************************************************************
##TOOL NAME: FitPlane
##SOURCE NAME: FitPlane.py
##VERSION: 2.0
##AUTHOR: B.M.J. Possel (b.m.j.possel@student.tudelft.nl
##REQUIRED ARGUMENTS: Input raster, Output folder
##
##TOOL DESCRIPTION: Manipulates a Raster map and writes them to a selected Folder.
##
##DATE: February 10, 2008
##
##Usage: FitPlane <Input_Raster_Dataset> <Output_Folder> 
##
##Code inspired by Matthew Perry's "slope.cpp" and Julien Fiore's "openness.py"
##*********************************************************************************************************************


# Import

import sys, string, os, time
try:
    import win32com.client, pythoncom
except ImportError:
    print "Install PyWin32: http://www.python.org/download/windows/"
try:
    from numpy import *
    from numpy.linalg import inv
except ImportError:
    print "Install Numpy: www.numpy.org"
try:
    from osgeo import gdal
except ImportError:
    print "Install gdal: www.gdal.org"
import padding          #Load padding script
#import FitPlaneprocess       #Load FitPlane script

msgSuccess="Successfully built file "
msgFail="Failed to build file "

gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")
gp.SetProduct("ArcView")

# Fitting plane using a least-squares approach
def FitPlane(T2, y2):
    xdak = dot(dot(inv(dot(T2.conj().transpose(),T2)),T2.conj().transpose()),y2)#Check if this is fully correct!!!
    return xdak

def CalculatePlane(InFile, OutFile1, OutFile2, size2, skippixel):
    # Open inGrid
    inGrid = gdal.Open(InFile)
    inGridBand = inGrid.GetRasterBand(1) # get raster band

    # Get inGrid parameters
    geoTrans = inGrid.GetGeoTransform() # returns list (xOrigin,xcellsize,...)

    cellSizeX = geoTrans[1]
    cellSizeY = geoTrans[5]
    nXSize = inGrid.RasterXSize
    nYSize = inGrid.RasterYSize
    
    #nullValue = inGridBand.GetNoDataValue() #For some reason it doesn't always work...
    nullValue2 = -999

    # Create  output file
    format = "GTiff"
    driver = gdal.GetDriverByName( format )
    outGrid1=driver.CreateCopy(OutFile1,inGrid)   # Creates output raster with same properties as input
    outGridBand1 = outGrid1.GetRasterBand(1)      # Access Band 1
    outGridBand1.SetNoDataValue(nullValue2)
    outGrid2=driver.CreateCopy(OutFile2,inGrid)   # Creates output raster with same properties as input
    outGridBand2 = outGrid2.GetRasterBand(1)      # Access Band 1
    outGridBand2.SetNoDataValue(nullValue2)

    min = inGridBand.GetMinimum()
    max = inGridBand.GetMaximum()
    #print 'Min=%.3f, Max=%.3f' % (min,max)

    # Matrix Manupulation
    matrix2 = inGridBand.ReadAsArray()
    nullValue = matrix2.min()
    [p,q] = shape(matrix2)              # Used for determining tile size

    [matrix, basemodel] = padding.padding(InFile,size2)                #Add padding from neighbours

    print 'matrix'
    print matrix[0,0]
    print matrix.dtype
    
    # Fitting planes
    [s,t] = shape(matrix)
    slope = ones((s,t),float)*nullValue2
    FitErrors = ones((s,t),float)*nullValue2

    #meshx = arange(0, (2*size2+2), 2)
    #meshy = arange(0, (2*size2+2), 2)
    #[X,Y] = meshgrid(meshx,meshy)

    n = range(1,(2*size2+2))
    [X,Y] = meshgrid(n,n)
    X2 = X[0:(len(X)):skippixel,0:(len(X)):skippixel]
    Y2 = Y[0:(len(Y)):skippixel,0:(len(Y)):skippixel]
    
    Xflat = X2.flatten(1)[:,newaxis]
    Yflat = Y2.flatten(1)[:,newaxis]
    #T = ones(((2*size2+1)**2,3),float)
    T = ones(((len(Xflat)),3),float)
    T[:,0] = Xflat[:,0]
    T[:,1] = Yflat[:,0]
    #T = T2[0:(size(T2)):2,:]

    xdakPartial = dot(inv(dot(T.conj().transpose(),T)),T.conj().transpose())
    
    for i in range(s)[(size2):(s-(size2+1))]:
        for j in range(t)[(size2):(t-(size2+1))]:
            if matrix[i,j] != nullValue:
                plane = matrix[(i-size2):(i+size2+1):skippixel,(j-size2):(j+size2+1):skippixel]
                planeflat = plane.flatten(1)[:,newaxis]
                if all(1==2) == True:
                #if all(planeflat[:]!=nullValue) == True:
                    xdak = dot(xdakPartial,planeflat[:,0])
                    slope[i,j] = arctan(sqrt(xdak[0,]**2+xdak[1,]**2))
                    
                else:
                    T[:,2] = planeflat[:,0]
                    TnoNaN = T[where(T[:,2]!=nullValue)[0],:]  # extract the columms where value = nullValue 
                    #TnoNaN is a float64 array -> why? usefull?

                    if len(TnoNaN) > 60:
                        y = TnoNaN[:,2].copy()
                        TnoNaN[:,2] = 1
                        xdak = FitPlane(TnoNaN,y)
                        slope[i,j] = arctan(sqrt(xdak[0,]**2+xdak[1,]**2))

                        #Calculate average error
                        #print TnoNaN
                        #print xdak
                        CalcValues = y - (TnoNaN[:,0]**xdak[0,] + TnoNaN[:,1]**xdak[1,] + xdak[2,])
                        #print CalcValues
                        MisFit = ((CalcValues**2).sum())/len(TnoNaN)
          #              MisFit = (CalcValues.conj().transpose()*CalcValues)/len(TnoNaN)
                        #print FitError
                        FitErrors[i,j] = abs(MisFit)
                        #time.sleep(5)

    print slope.max()
    print slope.min()

    # Writes buffer to file
    slope2 = slope[size2:(p+size2),size2:(q+size2)]
    FitErrors2 = FitErrors[size2:(p+size2),size2:(q+size2)]

    print 'matrix2'
    print slope2[0,0]
    print slope2.dtype
    time.sleep(5)
    
    outGridBand1.WriteArray(slope2)
    outGridBand1.FlushCache()
    outGridBand2.WriteArray(FitErrors2)
    outGridBand2.FlushCache()

#The input rasters
rasterlist = gp.GetParameterAsText(0)

#Remove the parathesis around each input featureclass
rasterlist = string.replace(rasterlist, "(", "")
rasterlist = string.replace(rasterlist, ")", "")

#split input raster datasets by semicolon ";"
in_raster_ds = string.split(rasterlist, ";")
    
for in_raster in in_raster_ds:
    raster = in_raster

    if raster.startswith("'") and raster.endswith("'"):
        raster = raster[1:-1]

    #Determine folder and filename
    OutFolder = gp.GetParameterAsText(1)
    OutFolder = string.replace(OutFolder, "\\", "/")
    OutFile1 = OutFolder + "/" + "slope_" + os.path.basename(raster)
    OutFile2 = OutFolder + "/" + "fiterror_" + os.path.basename(raster)
    raster = string.replace(raster, "\\", "/")
    #### TO ADD -> Als .adf niet bestaat, neem dan het input bestand!!!
    #InFile = raster + "/" + "w001001.adf"
    InFile = raster

    InFile = InFile.encode('utf8')
    OutFile1 = OutFile1.encode('utf8')
    OutFile2 = OutFile2.encode('utf8')

    size2 = int(gp.GetParameterAsText(2)) #Size of the plain
    if size2 < 1:
        print "Change Input Parameters: Size >= 1"
        time.sleep(5)
        sys.exit()

    skippixel = int(gp.GetParameterAsText(3)) #Amount of pixels to skip, uses every x-number of pixels
    if skippixel < 1:
        print "Change Input Parameters: Skip pixel >= 1"
        time.sleep(5)
        sys.exit()

##    slopethreshold = int(gp.GetParameterAsText(4)) #Threshold for maximum slope error
##    if slopethreshold < 1:
##        print "Change Input Parameters: slope error threshold >= 1"
##        time.sleep(5)
##        sys.exit()

    CalculatePlane(InFile,OutFile1,OutFile2,size2,skippixel)
    gp.AddMessage(msgSuccess + "%s" % raster)

        
#############################################################
