##********************************************************************************************************************
##TOOL NAME: Multi Classification
##SOURCE NAME: Multi_Classification.py
##VERSION: 1
##AUTHOR: B.M.J. Possel (b.m.j.possel@student.tudelft.nl
##REQUIRED ARGUMENTS: Input composite bands, Input Signature file, Output folder
##
##TOOL DESCRIPTION: Classifies multiple composite band files using a Maximum Likelihood
##                  Classification and writes the output to a selected Folder.
##
##DATE: March 2, 2009
##
##Usage: Multi_Classification <Input_Files> <Input_Signature_File> <Output_Folder>
##*********************************************************************************************************************

import sys, string, os, time
try:
    import win32com.client, pythoncom
except ImportError:
    print "Install PyWin32: http://www.python.org/download/windows/"
try:
    from numpy import *
    from numpy.linalg import inv
except ImportError:
    print "Install Numpy: www.numpy.org"
try:
    from osgeo import gdal
except ImportError:
    print "Install gdal: www.gdal.org"

msgSuccess="Successfully built file "
msgFail="Failed to build file "

gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")
gp.SetProduct("ArcView")

# Check out any necessary licenses
gp.CheckOutExtension("spatial")

# Load required toolboxes...
gp.AddToolbox("C:/Program Files/ArcGIS/ArcToolbox/Toolboxes/Spatial Analyst Tools.tbx")

#Get parameters
rasterlist = gp.GetParameterAsText(0)

SigFile = gp.GetParameterAsText(1)
SigFile = string.replace(SigFile, "\\", "/")

OutFolder = gp.GetParameterAsText(2)
OutFolder = string.replace(OutFolder, "\\", "/")

#Remove the parathesis around each input featureclass
rasterlist = string.replace(rasterlist, "(", "")
rasterlist = string.replace(rasterlist, ")", "")

#split input raster datasets by semicolon ";"
in_raster_ds = string.split(rasterlist, ";")

for in_raster in in_raster_ds:
    raster = in_raster

    if raster.startswith("'") and raster.endswith("'"):
        raster = raster[1:-1]

    raster = string.replace(raster, "\\", "/")

    #Determine folder and filename
    RasterName = string.split(raster, "_composite.tif")
    RasterName2 = RasterName[0]
    
    OutFile = OutFolder + "/" + os.path.basename(RasterName2) + "_classified.tif"

    #Convert names
    SigFile = SigFile.encode('utf8')
    OutFile = OutFile.encode('utf8')

    print raster
    print SigFile
    print OutFile

    # Process: MLClassify
    #gp.MLClassify_sa(raster, SigFile, OutFile)

    try:
        gp.MLClassify_sa(raster, SigFile, OutFile)
    except:
        # If an error occurred while running a tool, then print the messages.
        print gp.GetMessages()

    gp.AddMessage(msgSuccess + "%s" % raster)

        
#############################################################
