##********************************************************************************************************************
##TOOL NAME: Smoothing
##SOURCE NAME: Smooting.py
##VERSION: 1
##AUTHOR: B.M.J. Possel (b.m.j.possel@student.tudelft.nl
##REQUIRED ARGUMENTS: Input raster, Output folder, Windowsize
##
##TOOL DESCRIPTION: Smoothes a Raster map and writes it to a selected Folder.
##
##DATE: February 9, 2009
##
##Usage: Smoothing <Input_Raster_Dataset> <Output_Folder> <Window_Size>
##*********************************************************************************************************************

import sys, string, os, time
try:
    import win32com.client, pythoncom
except ImportError:
    print "Install PyWin32: http://www.python.org/download/windows/"
try:
    from numpy import *
    from numpy.linalg import inv
except ImportError:
    print "Install Numpy: www.numpy.org"
try:
    from osgeo import gdal
except ImportError:
    print "Install gdal: www.gdal.org"

msgSuccess="Successfully built file "
msgFail="Failed to build file "

gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")
gp.SetProduct("ArcView")

# Check out any necessary licenses
gp.CheckOutExtension("spatial")

# Load required toolboxes...
gp.AddToolbox("C:/Program Files/ArcGIS/ArcToolbox/Toolboxes/Spatial Analyst Tools.tbx")

#The input rasters
rasterlist = gp.GetParameterAsText(0)

#Remove the parathesis around each input featureclass
rasterlist = string.replace(rasterlist, "(", "")
rasterlist = string.replace(rasterlist, ")", "")

#split input raster datasets by semicolon ";"
in_raster_ds = string.split(rasterlist, ";")
    
for in_raster in in_raster_ds:
    raster = in_raster

    if raster.startswith("'") and raster.endswith("'"):
        raster = raster[1:-1]

    #Determine folder and filename
    OutFolder = gp.GetParameterAsText(1)
    OutFolder = string.replace(OutFolder, "\\", "/")
    OutFile = OutFolder + "/" + os.path.basename(raster) + "_smoothed.tif"
    raster = string.replace(raster, "\\", "/")
    #### TO ADD -> Als .adf niet bestaat, neem dan het input bestand!!!
    #InFile = raster + "/" + "w001001.adf"
    InFile = raster

    InFile = InFile.encode('utf8')
    OutFile = OutFile.encode('utf8')

    size2 = int(gp.GetParameterAsText(2)) #Size of the plain
    if size2 < 1:
        print "Change Input Parameters: Size >= 1"
        time.sleep(5)
        sys.exit()

    InNeighborhood = "Rectangle " + str(size2) + " " + str(size2) + " Cell"	
    InNoDataOption = "DATA"
    print InFile
    print OutFile
    print InNeighborhood

    # Process: FocalStatistics
    try:
        gp.FocalStatistics_sa(InFile, OutFile, InNeighborhood, "", InNoDataOption)
    except:
        # If an error occurred while running a tool, then print the messages.
        print gp.GetMessages()

    gp.AddMessage(msgSuccess + "%s" % raster)

        
#############################################################
