##********************************************************************************************************************
##TOOL NAME: TPI
##SOURCE NAME: TPI.py
##VERSION: 2.0
##AUTHOR: B.M.J. Possel (b.m.j.possel@student.tudelft.nl
##REQUIRED ARGUMENTS: Input raster, Output folder
##
##TOOL DESCRIPTION: Calculates TPI values from a Raster map and writes them to a selected Folder.
##
##DATE: December 3, 2008
##
##Usage: FitPlane <Input_Raster_Dataset> <Output_Folder> 
##
##Code inspired by Matthew Perry's "slope.cpp" and Julien Fiore's "openness.py"
##*********************************************************************************************************************

import sys, string, os
try:
    import win32com.client, pythoncom
except ImportError:
    print "Install PyWin32: http://www.python.org/download/windows/"
try:
    from numpy import *
except ImportError:
    print "Install Numpy: www.numpy.org"
try:
    from osgeo import gdal
except ImportError:
    print "Install gdal: www.gdal.org"
import padding          #Load padding script
import TPIprocess       #Load TPI script

msgSuccess="Successfully built file "
msgFail="Failed to build file "

gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")
gp.SetProduct("ArcView")
gp.CheckOutExtension("spatial")

def TPI(InFile, outfile, orad, irad):
    inGrid = gdal.Open(InFile)
    inGridBand = inGrid.GetRasterBand(1) # get raster band

    # Get parameters
    geoTrans = inGrid.GetGeoTransform() # returns list (xOrigin,xcellsize,...)
    cellSizeX = geoTrans[1]
    cellSizeY = geoTrans[5]
    nXSize = inGrid.RasterXSize
    nYSize = inGrid.RasterYSize
    #nullValue = inGridBand.GetNoDataValue() #For some reason it doesn't always work...
    nullValue2 = -999

    # Create  output file
    format = "GTiff"
    driver = gdal.GetDriverByName(format)
    outGrid=driver.CreateCopy(OutFile,inGrid)   # Creates output raster with
                                                # same properties as input
    outGridBand = outGrid.GetRasterBand(1)      # Access Band 1
    outGridBand.SetNoDataValue(nullValue2)      # Set value for point where there is no data

    min = inGridBand.GetMinimum()
    max = inGridBand.GetMaximum()
    #print 'Min=%.3f, Max=%.3f' % (min,max)

#test = inGridBand.GetRasterColorTable()

    # Put data into matrix
    matrix2 = inGridBand.ReadAsArray()  # Used for determining tile size
    nullValue = matrix2.min()
    [m,n] = shape(matrix2)              # Used for determining tile size

    
    print 'nullValue'
    print nullValue
    print type(nullValue)
    print 'nullValue2'
    print nullValue2
    print type(nullValue2)


    [matrix, basemodel] = padding.padding(InFile,orad)                #Add padding from neighbours

    print 'matrix'
    print matrix[0,0]
    print matrix.dtype

##    InFile = 'E:/Afstuderen_BMJ_Possel/Data/3._1x1.25km_g0/av07_38g014g0'
##    InExpression = 'focalmean('+InFile +', annulus, 150, 148, DATA)'
##    OutFile = 'E:/Afstuderen_BMJ_Possel/Data/3._1x1.25km_g0/temp/av07_38g014g0'
##    gp.SingleOutputMapAlgebra_sa(InExpression, OutFile)
##    outRaster = 'E:/Afstuderen_BMJ_Possel/Data/3._1x1.25km_g0/temp2/av07_38g014g0'
##    gp.Minus_sa(InFile, OutFile, outRaster)

    tpi = TPIprocess.TPIprocess(matrix, orad, irad, nullValue, nullValue2)        #Run TPI script
    tpi2 = tpi[orad:(m+orad),orad:(n+orad)]                                        #Select matrix for export to file

    print 'tpi'
    print tpi[0,0]
    print tpi.dtype

    outGridBand.WriteArray(tpi2)        # Writes buffer to file
    outGridBand.FlushCache()


#The input rasters
rasterlist = gp.GetParameterAsText(0)

#Remove the parathesis around each input featureclass
rasterlist = string.replace(rasterlist, "(", "")
rasterlist = string.replace(rasterlist, ")", "")

#split input raster datasets by semicolon ";"
in_raster_ds = string.split(rasterlist, ";")
    
for in_raster in in_raster_ds:
    raster = in_raster

    if raster.startswith("'") and raster.endswith("'"):
        raster = raster[1:-1]

    #Determine folder and filename
    OutFolder = gp.GetParameterAsText(1)
    OutFolder = string.replace(OutFolder, "\\", "/")
    OutFile = OutFolder + "/" + os.path.basename(raster) + "_tpi.tif"
    raster = string.replace(raster, "\\", "/")
    InFile = raster #+ "/" + "w001001.adf"

    InFile = InFile.encode('utf8')
    OutFile = OutFile.encode('utf8')

    orad = int(gp.GetParameterAsText(2)) #Outer Annulus Radius
    irad = int(gp.GetParameterAsText(3)) #Inner Annulus Radius
    if orad <= irad:                     #Check if Outer Annulus is larger than Inner Annulus
        print "Change Input Parameters: Outer Annulus Radius > Inner Annulus Radius"
        time.sleep(5)
        sys.exit()

    TPI(InFile,OutFile, orad, irad)      #Call main routine
    gp.AddMessage(msgSuccess + "%s" % raster)

        
#############################################################
