##********************************************************************************************************************
##TOOL NAME: Topology
##SOURCE NAME: Topology.py
##VERSION: 1
##AUTHOR: B.M.J. Possel (b.m.j.possel@student.tudelft.nl
##REQUIRED ARGUMENTS: Input rasters, Output folder
##
##TOOL DESCRIPTION: Calculates the topology of Raster maps and writes the result as multiple
##                  text files to the selected Folder.
##
##DATE: November 27, 2008
##
##Usage: Topology <Input_Raster_Dataset> <Output_Folder> 
##
##*********************************************************************************************************************

import sys, string, os
try:
    import win32com.client, pythoncom
except ImportError:
    print "Install PyWin32: http://www.python.org/download/windows/"
try:
    from numpy import *
except ImportError:
    print "Install Numpy: www.numpy.org"
try:
    from osgeo import gdal
except ImportError:
    print "Install gdal: www.gdal.org"

msgSuccess="Successfully built Topology file"
msgFail="Failed to build Topology file "

gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")
gp.SetProduct("ArcView")

def relations(rasternames,m,coordinates):
    tilecorners = [coordinates[m,2].copy(),coordinates[m,3].copy(),coordinates[m,6].copy(),coordinates[m,7].copy(),coordinates[m,0].copy(),coordinates[m,1].copy(),coordinates[m,4].copy(),coordinates[m,5].copy(),0]
    coordinates2 = abs(coordinates[:,:].copy()-tilecorners)          # absolute distance from specific corners of tile m
    coordinates2[m,:] = coordinates2[m,:]+9999999999999999           # alternative way to remove m from the coordinates???

    bottom = []
    right = []
    left = []
    top = []

    bottom = coordinates[where((coordinates2[:,0]<=10)&(coordinates2[:,1]<=10))[0],8]
    right = coordinates[where((coordinates2[:,2]<=10)&(coordinates2[:,3]<=10))[0],8]
    left = coordinates[where((coordinates2[:,4]<=10)&(coordinates2[:,5]<=10))[0],8]
    top = coordinates[where((coordinates2[:,6]<=10)&(coordinates2[:,7]<=10))[0],8]

    bottomtile = []
    righttile = []
    lefttile = []
    toptile = []
    
    if len(bottom) != 0:
        bottomtile = rasternames[int(bottom[0])]

    if len(right) != 0:
        righttile = rasternames[int(right[0])]

    if len(left) != 0:
        lefttile = rasternames[int(left[0])]

    if len(top) != 0:
        toptile = rasternames[int(top[0])]
    
    return bottomtile, righttile, lefttile, toptile


#The input rasters
rasterlist = gp.GetParameterAsText(0)
rasterlist = string.replace(rasterlist, "(", "")        #Remove the parathesis around each input featureclass
rasterlist = string.replace(rasterlist, ")", "")        #Remove the parathesis around each input featureclass
in_raster_ds = string.split(rasterlist, ";")            #split input raster datasets by semicolon ";"
in_raster_ds2 = in_raster_ds[:]                         #copy and flatten

#Get information to write file
OutFolder = gp.GetParameterAsText(1)
OutFolder = string.replace(OutFolder, "\\", "/")
OutFolder = OutFolder + "/Topology"
OutFile0 = OutFolder + "/0Raster.txt"
OutFile1 = OutFolder + "/1Above.txt"
OutFile2 = OutFolder + "/2Right.txt"
OutFile3 = OutFolder + "/3Below.txt"
OutFile4 = OutFolder + "/4Left.txt"
OutFolder = OutFolder.encode('utf8')
if not os.path.isdir(OutFolder + "/"):               #Determine if directory is already present
    os.mkdir(OutFolder + "/")                        #Make new directory

rasternames = []
upleft = []
lowleft = []
upright = []
lowright = []

coordinates = zeros((size(in_raster_ds),9),float)    #Create matrix for coordinates
coordinates[:,8] = range(0,(size(in_raster_ds)))     #Add numbering in last column

n = 0       #counter
for raster in in_raster_ds:
    #Remove first and last character
    if raster.startswith("'") and raster.endswith("'"):
        raster = raster[1:-1]
    #Determine filename and open file
    raster = string.replace(raster, "\\", "/")
    if os.path.isfile(raster + "/" + "w001001.adf"):
        InFile = raster + "/" + "w001001.adf"
    else:
        InFile = raster
    InFile = InFile.encode('utf8')
    rasternames.append(InFile)                      # add rasternames to list

    # Get parameters
    inGrid = gdal.Open(InFile)
    gtrn = inGrid.GetGeoTransform()

    #Next section only works for North-up images! -> http://www.gdal.org/classGDALDataset.html#f9593cc241e7d140f5f3c4798a43a668
    x_extent = gtrn[1] * (inGrid.RasterXSize - 1);
    y_extent = gtrn[5] * (inGrid.RasterYSize - 1);
    coordinates[n,0] = gtrn[0]                      # X upper left
    coordinates[n,1] = gtrn[3]                      # Y upper left
    coordinates[n,2] = gtrn[0]                      # X lower left
    coordinates[n,3] = gtrn[3] + y_extent           # Y lower left
    coordinates[n,4] = gtrn[0] + x_extent           # X upper right
    coordinates[n,5] = gtrn[3]                      # Y upper right
    coordinates[n,6] = gtrn[0] + x_extent           # X lower right
    coordinates[n,7] = gtrn[3] + y_extent           # Y lower right
    n = n + 1

m = 0       #counter
for raster in in_raster_ds:
    [bottom, right, left, top] = relations(rasternames,m,coordinates)
    upleft.append(top)
    lowleft.append(right)
    upright.append(bottom)
    lowright.append(left)
    m = m + 1

#Write data to files
OutFile0 = OutFile0.encode('utf8')
OutFile1 = OutFile1.encode('utf8')
OutFile2 = OutFile2.encode('utf8')
OutFile3 = OutFile3.encode('utf8')
OutFile4 = OutFile4.encode('utf8')

WriteFile0 = open ( OutFile0, 'w' )
WriteFile1 = open ( OutFile1, 'w' )
WriteFile2 = open ( OutFile2, 'w' )
WriteFile3 = open ( OutFile3, 'w' )
WriteFile4 = open ( OutFile4, 'w' )

WriteFile0.write( str(rasternames) )
WriteFile1.write( str(upleft) )
WriteFile2.write( str(lowleft) )
WriteFile3.write( str(upright) )
WriteFile4.write( str(lowright) )

WriteFile0.close()
WriteFile1.close()
WriteFile2.close()
WriteFile3.close()
WriteFile4.close()

gp.AddMessage(msgSuccess)
#############################################################
