##********************************************************************************************************************
##TOOL NAME: Dilate
##SOURCE NAME: Dilate.py
##VERSION: 1
##AUTHOR: B.M.J. Possel (b.m.j.possel@student.tudelft.nl
##REQUIRED ARGUMENTS: Input classified image, Input dilation size, Output folder
##
##TOOL DESCRIPTION: Dilation of binairy image (classified), the amount of dilation can be adjusted,
##                  writes the output to a selected Folder.
##
##DATE: March 2, 2009
##
##Usage: Dilate <Input_Files> <Output_Folder>
##
##Dilation code made by: Shawn Lankton
##*********************************************************************************************************************

import sys, string, os, time
try:
    import win32com.client, pythoncom
except ImportError:
    print "Install PyWin32: http://www.python.org/download/windows/"
try:
    from numpy import *
    from numpy.linalg import inv
except ImportError:
    print "Install Numpy: www.numpy.org"
try:
    from osgeo import gdal
except ImportError:
    print "Install gdal: www.gdal.org"

#-- dilates a 2D numpy array holding a binary image
def dilate(a,nullValue,win):
    #r = zeros(a.shape)
    print a
    print nullValue
    print win
    time.sleep(5)
    r = ones(a.shape)*nullValue
    [yy, xx] = where(a>0)

    # prepare neighboroods
    off = tile(range(-win,win+1),(2*win+1,1))
    x_off = off.flatten(1)
    y_off = off.T.flatten(1)

    # duplicate each neighborhood element for each index
    n = len(xx.flat)
    x_off = tile(x_off, (n,1)).flatten(1)
    y_off = tile(y_off, (n,1)).flatten(1)
    # round out offset
    ind = sqrt(x_off**2 + y_off**2) > win
    x_off[ind] = 0
    y_off[ind] = 0

    # duplicate each index for each neighborhood element
    xx = tile(xx, ((2*win+1)**2))
    yy = tile(yy, ((2*win+1)**2))

    nx = xx + x_off
    ny = yy + y_off

    # bounds checking
    ny[ny<0]=0
    ny[ny>a.shape[0]-1]=a.shape[0]-1
    nx[nx<0]=0
    nx[nx>a.shape[1]-1]=a.shape[1]-1
    r[ny,nx]=0;
    return r


def extract(matrix,Value):

    [s,t] = shape(matrix)
    OutRaster = zeros((s,t),float)
    #OutRaster = ones((s,t),float)

    for i in range(s)[0:(s-1)]:
        for j in range(t)[0:(t-1)]:
            if matrix[i,j] == Value:
                OutRaster[i,j] = Value
                
    return OutRaster

def fill(matrix2,matrix_0,matrixdilate,nullValue):
    [k,l] = shape(matrixdilate)

    for i in range(k)[0:(k-1)]:
        for j in range(l)[0:(l-1)]:
            if matrix2[i,j] == nullValue and matrixdilate[i,j] == 0:
                matrix_0[i,j] = 1

    return matrix_0

msgSuccess="Successfully built file "
msgFail="Failed to build file "

gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")
gp.SetProduct("ArcView")

#Get parameters
rasterlist = gp.GetParameterAsText(0)

DilateSize = int(gp.GetParameterAsText(1))

OutFolder = gp.GetParameterAsText(2)
OutFolder = string.replace(OutFolder, "\\", "/")

#Remove the parathesis around each input featureclass
rasterlist = string.replace(rasterlist, "(", "")
rasterlist = string.replace(rasterlist, ")", "")

#split input raster datasets by semicolon ";"
in_raster_ds = string.split(rasterlist, ";")

for in_raster in in_raster_ds:
    raster = in_raster

    if raster.startswith("'") and raster.endswith("'"):
        raster = raster[1:-1]

    raster = string.replace(raster, "\\", "/")

    #Determine folder and filename
    RasterName = string.split(raster, ".tif")
    RasterName2 = RasterName[0]
    
    OutFile = OutFolder + "/" + os.path.basename(RasterName2) + "_dilate.tif"
    OutFile = OutFile.encode('utf8')
    InFile = raster.encode('utf8')

    print raster
    print DilateSize
    print OutFile

    inGrid = gdal.Open(InFile)
    inGridBand = inGrid.GetRasterBand(1)

    geoTrans = inGrid.GetGeoTransform() # returns list (xOrigin,xcellsize,...)

    cellSizeX = geoTrans[1]
    cellSizeY = geoTrans[5]
    nXSize = inGrid.RasterXSize
    nYSize = inGrid.RasterYSize
      
    matrix2 = inGridBand.ReadAsArray()

    #nullValue = inGridBand.GetNoDataValue() #For some reason it doesn't always work...
    nullValue = int(matrix2.max())
    print nullValue
        
    #Perform Dilation!

    matrix_0= extract(matrix2,1)
    print matrix2
    print matrix_0
    matrixdilate = dilate(matrix_0,nullValue,DilateSize)

    matrixfill = fill(matrix2,matrix_0,matrixdilate,nullValue)

    # Create  output file
    format = "GTiff"
    driver = gdal.GetDriverByName( format )
    outGrid=driver.CreateCopy(OutFile,inGrid)   # Creates output raster with same properties as input
    outGridBand = outGrid.GetRasterBand(1)      # Access Band 1
##    outGridBand.SetNoDataValue(-nullValue)
    outGridBand.SetNoDataValue(0)

    outGridBand.WriteArray(matrixfill)
    outGridBand.FlushCache()


    gp.AddMessage(msgSuccess + "%s" % raster)
        
#############################################################
